/* MasterClient.java
--------------------------------------------------------------------------------
Retrieves a list of servers.  To use create a new MasterClient and call the
refresh method.  Finally close the connection with the close function.

You should retrieve a list of client.Server that are sorted to their IP.  The
refresh() function can return null if it hasn't been 10 seconds or the user is
banned.
*/

package client;

import client.*;
import huffman.*;

import java.io.*;
import java.net.*;
import java.util.*;

import static main.Main.unsigned;
import static main.Main.error;

public class MasterClient
{
	private static final int MAX_SERVERS = 1000;
	public static final int RESPONSE_OK = 0;
	public static final int RESPONSE_BANNED = 3;
	public static final int RESPONSE_DENIED = 4;

	private DatagramSocket socket;
	private InetAddress address;
	private int port;
	private int lastResponse;

	public MasterClient(String address, int port)
	{
		try
		{
			this.address = InetAddress.getByName(address);
			this.port = port;
			socket = new DatagramSocket();
			socket.setSoTimeout(10000);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	protected void finalize() //Close on garbage collection
	{
		close();
	}
	public void close()
	{
		socket.close();
	}
	public int getLastResponse()
	{
		return lastResponse;
	}
	public Server[] refresh() throws IOException
	{
		LinkedList<Server> servers = new LinkedList<Server>();
		DatagramPacket packet;
		byte[] data = {(byte) 0xC7, (byte) 0x00, (byte) 0x00, (byte) 0x00};
		socket.send(new DatagramPacket(Huffman.encode(data), 4, address, port));
		byte[] pack = new byte[5 + (7*MAX_SERVERS)];
		packet = new DatagramPacket(pack, pack.length);
		socket.receive(packet);
		pack = Huffman.decode(pack);
		lastResponse = pack[0];
		if(lastResponse == RESPONSE_OK)
		{
			int pos = 4;
			while(true)
			{
				if(pack[pos] == 2) //End of list
					break;
				byte[] addr = {pack[pos+1], pack[pos+2], pack[pos+3], pack[pos+4]};
				servers.add(new Server(InetAddress.getByAddress(addr), unsigned(pack[pos+5]) + unsigned(pack[pos+6])*0x100));
				pos += 7;
			}
		}
		else if(lastResponse == RESPONSE_BANNED)
		{
			error("Your IP has been banned from the master server!");
			return null;
		}
		else if(lastResponse == RESPONSE_DENIED)
		{
			error("Please wait 10 seconds because refreshing.");
			return null;
		}
		Server[] list = servers.toArray(new Server[0]);
		Arrays.sort(list); //Order by IP
		return list;
	}
}