/* Server.java
--------------------------------------------------------------------------------
Is designed to hold information for one unique server.  Additionally it houses
enums for DMFlags, DMFlags2, and Compatibility Flags.
*/

package client;

import main.*;
import main.wad.*;
import gui.*;
import huffman.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;

import static main.Main.unsigned;

public class Server implements Comparable<Server>, ImageObserver
{
	public enum DMFlag
	{
		DISALLOW_HEALTH(1), DISALLOW_POWERUPS(2), WEAPONS_STAY(4),
		FALLING_DAMAGE_OLD(8), FALLING_DAMAGE_HEXEN(16), SAME_MAP(64),
		SPAWN_FATHEST(128), FORCE_RESPAWN(256), DISALLOW_ARMOR(512),
		DISALLOW_EXIT(1024), INFINITE_AMMO(2048), NO_MONSTERS(4096),
		MONSTERS_RESPAWN(8192), ITEMS_RESPAWN(16384), FAST_MONSTERS(32768),
		DISALLOW_JUMP(65536), DISALLOW_FREELOOK(131072),
		MEGA_POWERUPS_RESPAWN(262144), DISALLOW_FOV(524288),
		DONT_SPAWN_MULTI_WEAPONS(1048576), DISALLOW_CROUCH(2097152),
		LOSE_INVENTORY(4194304), LOSE_KEYS(8388608), LOSE_WEAPONS(16777216),
		LOSE_ARMOR(33554432), LOSE_POWERUPS(67108864), LOSE_AMMO(134217728),
		LOSE_HALF_AMMO(268435456);

		public final int value;

		DMFlag(int value)
		{
			this.value = value;
		}

		//Convert numberic DMFlag value to enum
		public static DMFlag[] getFlags(int dmflags)
		{
			//Populate list of flags
			LinkedList<DMFlag> flags = new LinkedList<DMFlag>();
			for(DMFlag flag : DMFlag.values())
				if((dmflags & flag.value) > 0) flags.add(flag);

			//Convert to DMFlag array and then return the array
			DMFlag[] array = new DMFlag[flags.size()];
			for(int i = 0;i < flags.size();i++)
				array[i] = flags.get(i);
			return array;
		}
		//Convert DMFlag array to a number
		public static int flagsToInteger(DMFlag[] flags)
		{
			int returnValue = 0;
			for(DMFlag flag : flags)
				returnValue += flag.value;
			return returnValue;
		}
		//Remove underscores in the toString function
		public String toString()
		{
			return super.toString().replace('_', ' ');
		}
	}

	public enum DMFlag2
	{
		DROP_WEAPONS(2), DONT_SPAWN_RUNES(4), INSTANT_FLAG_RETURN(8),
		NO_TEAM_SWITCHING(16), SERVER_PICKS_TEAMS(32), DOUBLE_AMMO(64),
		DEGENERATION(128), ALLOW_BFG_FREEAIMING(256), BARRELS_RESPAWN(512),
		NO_RESPAWN_PROTECTION(1024), START_WITH_SHOTGUN(2048),
		SPAWN_WHERE_DIED(4096), KEEP_TEAM_AFTER_A_MAP_CHANGE(8192);

		public final int value;

		DMFlag2(int value)
		{
			this.value = value;
		}

		//Convert numberic DMFlag2 value to enum
		public static DMFlag2[] getFlags(int dmflags)
		{
			//Populate list of flags
			LinkedList<DMFlag2> flags = new LinkedList<DMFlag2>();
			for(DMFlag2 flag : DMFlag2.values())
				if((dmflags & flag.value) > 0) flags.add(flag);

			//Convert to DMFlag2 array and then return the array
			DMFlag2[] array = new DMFlag2[flags.size()];
			for(int i = 0;i < flags.size();i++)
				array[i] = flags.get(i);
			return array;
		}
		//Convert DMFlag2 array to a number
		public static int flagsToInteger(DMFlag2[] flags)
		{
			int returnValue = 0;
			for(DMFlag2 flag : flags)
				returnValue += flag.value;
			return returnValue;
		}
		//Remove underscores in the toString function
		public String toString()
		{
			return super.toString().replace('_', ' ');
		}
	}

	public enum CompatFlag
	{
		FIND_SHORTEST_TEXTURES_LIKE_DOOM(1), USE_BUGGIER_STAIR_BUILDING(2),
		LIMIT_PAIN_ELEMENTALS_TO_20_LOST_SOULS(4),
		DONT_LET_OTHERS_HEAR_PICKUPS(8), ACTORS_ARE_INFINITELY_TALL(16),
		ALLOW_SILENT_BFG_TRICK(32), ENABLE_WALL_RUNNING(64),
		SPAWN_ITEM_DROPS_ON_THE_FLOOR(128),
		ALL_SPECIAL_LINES_CAN_DROP_USE_LINES(256),
		DISABLE_BOOM_DOOR_LIGHT_EFFECT(512),
		RAVENS_SCROLLERS_USE_ORIGINAL_SPEED(1024),
		USE_SECTOR_BASED_SOUND_TARGET_CODE(2048),
		LIMIT_DEH_MAXHEALTH_TO_HEALTH_BONUS(4096),
		TRACE_IGNORE_LINES_WITH_SAME_SECTOR_ON_BOTH_SIDES(8192),
		NO_MONSTER_DROPOFF_MOVE(16384), SCROLLING_SECTORS_ARE_ADDITIVE(32768),
		MONSTERS_SEE_SEMIINVISIBLE_PLAYERS(65536),
		LIMITED_MOVEMENT_IN_THE_AIR(131072), PLASMA_BUMP_BUG(262144),
		ALLOW_INSTANT_RESPAWN(524288), DISABLE_TAUNTS(1048576),
		ORIGINAL_SOUND_CURVE(2097152), USE_OLD_INTERMISSION_SCREENS(4194304),
		DISABLE_STEALTH_MONSTERS(8388608);

		public final int value;

		CompatFlag(int value)
		{
			this.value = value;
		}

		//Convert numberic CompatFlag value to enum
		public static CompatFlag[] getFlags(int compatflags)
		{
			//Populate list of flags
			LinkedList<CompatFlag> flags = new LinkedList<CompatFlag>();
			for(CompatFlag flag : CompatFlag.values())
				if((compatflags & flag.value) > 0) flags.add(flag);

			//Convert to CompatFlag array and then return the array
			CompatFlag[] array = new CompatFlag[flags.size()];
			for(int i = 0;i < flags.size();i++)
				array[i] = flags.get(i);
			return array;
		}
		//Convert CompatFlag array to a number
		public static int flagsToInteger(CompatFlag[] flags)
		{
			int returnValue = 0;
			for(CompatFlag flag : flags)
				returnValue += flag.value;
			return returnValue;
		}
		//Remove underscores in the toString function
		public String toString()
		{
			return super.toString().replace('_', ' ');
		}
	}

	public enum GameMode
	{
		COOPERATIVE(0, "Cooperative", "Cooperative", false),
		SURVIVAL(1, "Survival", "Survival", false),
		INVASION(2, "Invasion", "Invasion", false),
		DEATHMATCH(3, "Deathmatch", "", false),
		TEAMPLAY(4, "Team Deathmatch", "Team", true),
		DUEL(5, "Duel", "Duel", false),
		TERMINATOR(6, "Terminator", "Terminator", false),
		LASTMANSTANDING(7, "LMS", "LMS", false),
		TEAMLMS(8, "Team LMS", "Team LMS", true),
		POSSESSION(9, "Possession", "Poss.", false),
		TEAMPOSSESSION(10, "Team Possession", "Team Poss.", true),
		TEAMGAME(11, "Team Game", "Team Game", true),
		CTF(12, "CTF", "CTF", true),
		ONEFLAGCTF(13, "One Flag CTF", "One Flag CTF", true),
		SKULLTAG(14, "Skulltag", "Skulltag", true);

		public final int value;
		public final String name;
		public final String name_root;
		public final boolean teamGame;

		GameMode(int value, String name, String name_root, boolean teamGame)
		{
			this.value = value;
			this.name = name;
			this.name_root = name_root;
			this.teamGame = teamGame;
		}

		public static GameMode getGameMode(int mode)
		{
			for(GameMode gm : GameMode.values())
			{
				if(gm.value == mode)
					return gm;
			}
			return COOPERATIVE;
		}
		public String toString()
		{
			return name;
		}
		public String toString(boolean instagib, boolean buckshot)
		{
			if(name_root.equals("") && (instagib || buckshot))
			{
				if(instagib)
					return "Instagib";
				else if(buckshot)
					return "Buckshot";
			}
			if(instagib)
				return (name_root + " IG");
			else if(buckshot)
				return (name_root + " BShot");
			return name;
		}
		public boolean isTeamGame()
		{
			return teamGame;
		}
	}

	public static final int SQF_NAME =				0x00000001;
	public static final int SQF_URL =				0x00000002;
	public static final int SQF_EMAIL =				0x00000004;
	public static final int SQF_MAPNAME =			0x00000008;
	public static final int SQF_MAXCLIENTS =		0x00000010;
	public static final int SQF_MAXPLAYERS =		0x00000020;
	public static final int SQF_PWADS =				0x00000040;
	public static final int SQF_GAMETYPE =			0x00000080;
	public static final int SQF_GAMENAME =			0x00000100;
	public static final int SQF_IWAD =				0x00000200;
	public static final int SQF_FORCEPASSWORD =		0x00000400;
	public static final int SQF_FORCEJOINPASSWORD =	0x00000800;
	public static final int SQF_GAMESKILL =			0x00001000;
	public static final int SQF_BOTSKILL =			0x00002000;
	public static final int SQF_DMFLAGS =			0x00004000;
	public static final int SQF_LIMITS =			0x00010000;
	public static final int SQF_TEAMDAMAGE =		0x00020000;
	public static final int SQF_TEAMSCORES =		0x00040000;
	public static final int SQF_NUMPLAYERS =		0x00080000;
	public static final int SQF_PLAYERDATA =		0x00100000;
	public static final int SQF_ALL =				0x001F7FFF;

	public static final int STATUS_GOOD =	5660023;
	public static final int STATUS_DENIED =	5660024;
	public static final int STATUS_BANNED =	5660025;

	public static final int TEAM_BLUE = 0;
	public static final int TEAM_RED = 0;

	public static final char[] colorChar = {'\034'};

	protected final InetAddress address;
	protected final int port;
	private boolean responsive = false;
	private boolean banned = false;
	private ByteArrayInputStream packStream;

	private int ping;
	private String version;
	private long lastFlags;

	private String name = "Unknown Server";
	private String url = "";
	private String email = "";
	private String map = "";
	private String gameName = "ERROR!";
	private String iwad = "";
	private short maxClients = 0;
	private short maxPlayers = 0;
	private short gameSkill = 0;
	private short botSkill = 0;
	private boolean serverPassword = false;
	private boolean joinPassword = false;

	private short numWads = 0;
	private ArrayList<WadFile> wads = new ArrayList<WadFile>();

	private GameMode gameMode = GameMode.COOPERATIVE;
	private boolean instagib = false;
	private boolean buckshot = false;

	private DMFlag[] dmflags;
	private DMFlag2[] dmflags2;
	private CompatFlag[] compatflags;

	private int fragLimit = 0;
	private int timeLimit = 0;
	private int timeLeft = 0;
	private int duelLimit = 0;
	private int pointLimit = 0;
	private int winLimit = 0;
	private int[] teamScore = new int[2];

	private short numPlayers = 0;
	private ArrayList<Player> players = new ArrayList<Player>();

	//Images
	private static Image[] playerImages = new Image[6];
	private static final int IMAGE_SLOTOPEN = 0;
	private static final int IMAGE_SPECOPEN = 1;
	private static final int IMAGE_PLAYER = 2;
	private static final int IMAGE_BLUEPLAYER = 3;
	private static final int IMAGE_REDPLAYER = 4;
	private static final int IMAGE_SPECTATOR = 5;

	public Server(InetAddress address, int port)
	{
		this.address = address;
		this.port = port;
		if(playerImages[0] == null)
		{
			Toolkit tk = Toolkit.getDefaultToolkit();
			try
			{
				playerImages[0] = tk.createImage(getClass().getResource("/openslot.png"));
				playerImages[1] = tk.createImage(getClass().getResource("/openspec.png"));
				playerImages[2] = tk.createImage(getClass().getResource("/noteam.png"));
				playerImages[3] = tk.createImage(getClass().getResource("/blueteam.png"));
				playerImages[4] = tk.createImage(getClass().getResource("/redteam.png"));
				playerImages[5] = tk.createImage(getClass().getResource("/fillspec.png"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	public InetAddress inetAddress()
	{
		return address;
	}
	public String address()
	{
		byte[] addr = address.getAddress();
		return unsigned(addr[0]) + "." + unsigned(addr[1]) + "." + unsigned(addr[2]) + "." + unsigned(addr[3]) + ":" + port();
	}
	public int port()
	{
		return port;
	}
	public String name()
	{
		return name;
	}
	public String url()
	{
		return url;
	}
	public int ping()
	{
		return ping;
	}
	public String map()
	{
		return map.toLowerCase();
	}
	public String game()
	{
		if(iwad.lastIndexOf(".") != -1)
			return iwad.substring(0, iwad.lastIndexOf(".")).toLowerCase();
		return iwad.toLowerCase();
	}
	public String wadList()
	{
		String list = "";
		boolean first = true;
		for(WadFile file : wads)
		{
			if(!first)
				list += ", ";
			first = false;
			if(file.fileName().lastIndexOf(".") != -1)
				list += file.fileName().substring(0, file.fileName().lastIndexOf("."));
			else
				list += file.fileName();
		}
		return list.toLowerCase();
	}
	public String modeName()
	{
		return gameMode.toString(instagib, buckshot);
	}
	public java.util.List<String> launchString() throws IWadNotFoundException, PWadNotFoundException
	{
		LinkedList<String> arguments = new LinkedList<String>();
		arguments.add(Config.skulltagPath);
		WadFile iwadFile = new WadFile(iwad);
		arguments.add("+connect");
		arguments.add(address());
		arguments.add("-iwad");
		arguments.add(iwadFile.getLocation(true));
		if(wads.size() > 0)
		{
			arguments.add("-file");
			for(WadFile wad : wads)
				arguments.add(wad.getLocation(false));
		}
		return arguments;
	}
	public int numberOfPlayers(boolean noBots)
	{
		if(noBots)
		{
			int num = 0;
			for(Player player : players)
			{
				if(!player.isBot())
					num++;
			}
			return num;
		}
		return numPlayers;
	}
	public int maxClients()
	{
		return maxClients;
	}
	public int maxPlayers()
	{
		return maxPlayers;
	}
	public boolean isResponsive()
	{
		return responsive;
	}
	public String getPlayersToolTip()
	{
		String value = "";
		boolean first = true;
		for(Player player : players)
		{
			if(!first)
				value += ", ";
			value += player.name();
			first = false;
		}
		if(value.equals(""))
			return null;
		return value;
	}
	public PlayersImage getPlayersImage()
	{
		int playerCount = 0;
		int bluePlayers = 0;
		int redPlayers = 0;
		int spectators = 0;
		for(Player player : players)
		{
			if(player.isSpectating())
			{
				spectators++;
				continue;
			}
			if(gameMode.isTeamGame())
			{
				if(player.team(TEAM_BLUE))
				{
					bluePlayers++;
					continue;
				}
				else if(!player.team(255))
				{
					redPlayers++;
					continue;
				}
			}
			playerCount++;
		}
		int pslotsWithSpectators = -((maxClients-maxPlayers)-spectators);
		if(pslotsWithSpectators < 0)
			pslotsWithSpectators = 0;
		//System.out.println("players: " + playerCount + " blue: " + bluePlayers + " red: " + redPlayers + " specs: " + spectators + " fillingSpecs: " + pslotsWithSpectators);
		PlayersImage im = new PlayersImage(7*maxClients + 1, 16, BufferedImage.TYPE_INT_ARGB);
		Graphics g = im.getGraphics();
		im.setPlayers(spectators + playerCount + bluePlayers + redPlayers);
		int slot = -1;
		for(int i = maxClients-1;i >= 0;i--)
		{
			slot++;
			int x = i*7;
			if(playerCount > 0)
			{
				g.drawImage(playerImages[IMAGE_PLAYER], x, 0, this);
				playerCount--;
				continue;
			}
			if(bluePlayers > 0)
			{
				g.drawImage(playerImages[IMAGE_BLUEPLAYER], x, 0, this);
				bluePlayers--;
				continue;
			}
			if(redPlayers > 0)
			{
				g.drawImage(playerImages[IMAGE_REDPLAYER], x, 0, this);
				redPlayers--;
				continue;
			}
			if(slot < (maxPlayers-pslotsWithSpectators)) //draw open unless spectators are filling non spec slots
			{
				g.drawImage(playerImages[IMAGE_SLOTOPEN], x, 0, this);
				continue;
			}
			if(spectators > 0)
			{
				g.drawImage(playerImages[IMAGE_SPECTATOR], x, 0, this);
				spectators--;
				continue;
			}
			g.drawImage(playerImages[IMAGE_SPECOPEN], x, 0, this);
		}
		return im;
	}
	public int compareTo(Server o) //Compares the IP addresses
	{
		byte[] ip1 = address.getAddress();
		byte[] ip2 = o.address.getAddress();
		for(int i = 0;i < 4;i++)
		{
			if(ip1[i] < ip2[i])
				return -1;
			if(ip1[i] > ip2[i])
				return 1;
		}
		if(port < o.port)
			return -1;
		if(port > o.port)
			return 1;
		return 0;
	}

	public int refresh(int flags)
	{
		try
		{
			DatagramSocket sock = new DatagramSocket();
			sock.setSoTimeout(Config.waitingForServerTimeOut);
			byte[] data = putLong(new byte[0], 199);
			data = putLong(data, flags);
			data = putLong(data, System.currentTimeMillis());
			data = Huffman.encode(data);
			long time = System.currentTimeMillis(); //we won't be using the long for time since we can be more accurate here.
			sock.send(new DatagramPacket(data, data.length, address, port));
			byte[] pack = new byte[1200];
			DatagramPacket packet = new DatagramPacket(pack, pack.length);
			sock.receive(packet);
			ping = (int) (System.currentTimeMillis() - time); //do this before anything else for best results
			try
			{
				pack = Huffman.decode(Arrays.copyOfRange(pack, 0, packet.getLength()));
			}
			catch(ArrayIndexOutOfBoundsException e) {}
			responsive = true;
			sock.close();
			int status = unsigned(pack[0]) + unsigned(pack[1])*0x100 + unsigned(pack[2])*0x10000 + unsigned(pack[3])*0x1000000;
			if(status != STATUS_GOOD)
			{
				if(status == STATUS_DENIED)
					return STATUS_DENIED;
				else
					return STATUS_BANNED;
			}
			packStream = new ByteArrayInputStream(pack); //start up a stream for easy reading
			packStream.skip(8); //skip over the status and time

			//Global Values
			version = getString();
			lastFlags = getLong();
			if(Config.debianFix && (lastFlags > SQF_ALL || lastFlags < 0))
				lastFlags = flags; //Stange error...

			if(checkFlag(SQF_NAME))
				name = getString();
			if(checkFlag(SQF_URL))
				url = getString();
			if(checkFlag(SQF_EMAIL))
				email = getString();
			if(checkFlag(SQF_MAPNAME))
				map = getString();
			if(checkFlag(SQF_MAXCLIENTS))
				maxClients = getByte();
			if(checkFlag(SQF_MAXPLAYERS))
			{
				maxPlayers = getByte();
				if(maxClients != 0 &&  maxPlayers > maxClients) //The server is improperly set up so lets fix the data.
					maxPlayers = maxClients;
			}
			if(checkFlag(SQF_PWADS))
			{
				numWads = getByte();
				wads = new ArrayList<WadFile>();
				for(int i = 0;i < numWads;i++)
				{
					wads.add(new WadFile(getString()));
				}
			}
			if(checkFlag(SQF_GAMETYPE))
			{
				if(Config.debianFix && lastFlags == SQF_GAMETYPE)
					getByte();
				gameMode = GameMode.getGameMode(getByte());
				if(getByte() != 0)
					instagib = true;
				if(getByte() != 0)
					buckshot = true;
			}
			if(checkFlag(SQF_GAMETYPE)) //I don't know why this happens
			{
				if(checkFlag(SQF_GAMENAME))
					iwad = getString();
				if(checkFlag(SQF_IWAD))
					gameName = getString();
			}
			else
			{
				if(checkFlag(SQF_GAMENAME))
					gameName = getString();
				if(checkFlag(SQF_IWAD))
					iwad = getString();
			}
			if(checkFlag(SQF_FORCEPASSWORD) && getByte() != 0)
				serverPassword = true;
			if(checkFlag(SQF_FORCEJOINPASSWORD) && getByte() != 0)
				joinPassword = true;
			if(checkFlag(SQF_GAMESKILL))
				gameSkill = getByte();
			if(checkFlag(SQF_BOTSKILL))
				botSkill = getByte();
			if(checkFlag(SQF_DMFLAGS))
			{
				dmflags = DMFlag.getFlags((int) getLong());
				dmflags2 = DMFlag2.getFlags((int) getLong());
				compatflags = CompatFlag.getFlags((int) getLong());
			}
			if(checkFlag(SQF_LIMITS))
			{
				fragLimit = getShort();
				timeLimit = getShort();
				timeLeft = getShort();
				duelLimit = getShort();
				pointLimit = getShort();
				winLimit = getShort();
			}
			if(checkFlag(SQF_TEAMDAMAGE))
				getLong();
			if(checkFlag(SQF_TEAMSCORES))
			{
				teamScore[TEAM_BLUE] = getShort();
				teamScore[TEAM_RED] = getShort();
			}
			if(checkFlag(SQF_NUMPLAYERS))
				numPlayers = getByte();
			if(checkFlag(SQF_PLAYERDATA))
			{
				players = new ArrayList<Player>();
				for(int i = 0;i < numPlayers;i++)
					players.add(new Player(getString().replaceAll(new String(colorChar) + "[a-zA-Z+\\-]{1}", ""), getShort(), getShort(), getByte(), getByte(), getByte(), getByte()));
			}
		}
		catch(IncompletePacketException e)
		{
			responsive = true;
			ping = 999;
		}
		catch(SocketTimeoutException e)
		{
			responsive = false;
			ping = 999;
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return STATUS_GOOD;
	}
	private byte[] putLong(byte[] packet, long in) //appends the long to the end of the packet
	{
		byte[] pack = Arrays.copyOf(packet, packet.length+4);
		int pos = packet.length;
		String value = Long.toHexString(in);
		while(value.length() < 8)
			value = "0" + value;
		pack[pos] = (byte) Short.parseShort(value.substring(6, 8), 16);
		pack[pos+1] = (byte) Short.parseShort(value.substring(4, 6), 16);
		pack[pos+2] = (byte) Short.parseShort(value.substring(2, 4), 16);
		pack[pos+3] = (byte) Short.parseShort(value.substring(0, 2), 16);
		return pack;
	}
	private boolean checkFlag(int flag)
	{
		if((lastFlags & flag) == flag)
			return true;
		return false;
	}
	private String getString() throws IncompletePacketException
	{
		String out = new String("");
		byte[] value = {0}; //Array for string assembly
		if(packStream.available() < 1)
			throw new IncompletePacketException();
		while((value[0] = (byte) packStream.read()) != 0x00)
		{
			out += new String(value);
			if(packStream.available() < 1)
				throw new IncompletePacketException();
		}
		return out;
	}
	private long getLong() throws IncompletePacketException
	{
		if(packStream.available() < 4)
			throw new IncompletePacketException();
		byte[] buf = new byte[4];
		packStream.read(buf, 0, 4);
		return buf[0] + buf[1]*0x100 + buf[2]*0x10000 + buf[3]*0x1000000;
	}
	private int getShort() throws IncompletePacketException
	{
		if(packStream.available() < 2)
			throw new IncompletePacketException();
		byte[] buf = new byte[2];
		packStream.read(buf, 0, 2);
		return buf[0] + buf[1]*0x100;
	}
	private short getByte() throws IncompletePacketException
	{
		if(packStream.available() < 1)
			throw new IncompletePacketException();
		return (short) packStream.read();
	}

	//ImageObserver
	public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height){ return true; }
}