/* ConfigDialog.java
--------------------------------------------------------------------------------
Allows the user to configure the application without manually editing the cfg.
*/

package gui;

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import javax.swing.*;
import javax.swing.border.*;

import main.*;

public class ConfigDialog extends WindowAdapter implements ActionListener, WindowListener
{
	private static boolean open = false;
	private static JFileChooser fc = new JFileChooser();
	private JFrame frame = new JFrame("Configure");
	private JTextField program, download, master;
	private JFormattedTextField masterPort;
	private JList iwadList, pwadList;
	private JSpinner timeout;
	private JCheckBox adaptToSystem, debianFix;

	public ConfigDialog()
	{
		if(open)
			return;
		open = true;

		Dimension size = new Dimension(640, 400);
		frame.setMinimumSize(size);
		frame.setResizable(false);
		frame.addWindowListener(this);

		SpringLayout sl = new SpringLayout();
		JPanel paths = new JPanel(sl);
		paths.setBorder(new TitledBorder(new EtchedBorder(), "Paths"));
		{
			//Init all components that we will be using
			JLabel programLabel = new JLabel("Skulltag Path:");
			program = new JTextField(11);
			JButton programButton = new JButton("Browse");
			programButton.addActionListener(this);
			programButton.setActionCommand("Browse skulltagPath");
			iwadList = new JList();
			iwadList.setVisibleRowCount(3);
			iwadList.setFixedCellWidth(280);
			iwadList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			JLabel iwadLabel = new JLabel("IWAD Path(s):");
			JScrollPane iwadScrollPane = new JScrollPane(iwadList);
			JButton addIwad = new JButton("Add");
			addIwad.addActionListener(this);
			addIwad.setActionCommand("Add iwadPath");
			JButton removeIwad = new JButton("Remove");
			removeIwad.addActionListener(this);
			removeIwad.setActionCommand("Remove iwadPath");
			pwadList = new JList();
			pwadList.setVisibleRowCount(4);
			pwadList.setFixedCellWidth(280);
			pwadList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			JLabel pwadLabel = new JLabel("PWAD Path(s):");
			JScrollPane pwadScrollPane = new JScrollPane(pwadList);
			JButton addPwad = new JButton("Add");
			addPwad.addActionListener(this);
			addPwad.setActionCommand("Add pwadPath");
			JButton removePwad = new JButton("Remove");
			removePwad.addActionListener(this);
			removePwad.setActionCommand("Remove pwadPath");
			JLabel downloadLabel = new JLabel("Download PWADs to:");
			download = new JTextField(18);
			JButton downloadButton = new JButton("Browse");
			downloadButton.addActionListener(this);
			downloadButton.setActionCommand("DirBrowse downloadDir");

			//Adjust the layout
			sl.putConstraint(SpringLayout.WEST, programLabel, 2, SpringLayout.WEST, paths);
			sl.putConstraint(SpringLayout.NORTH, programLabel, 5, SpringLayout.NORTH, paths);
			sl.putConstraint(SpringLayout.WEST, program, 2, SpringLayout.EAST, programLabel);
			sl.putConstraint(SpringLayout.NORTH, program, 0, SpringLayout.NORTH, programLabel);
			sl.putConstraint(SpringLayout.WEST, programButton, 2, SpringLayout.EAST, program);
			sl.putConstraint(SpringLayout.NORTH, programButton, -3, SpringLayout.NORTH, program);
			sl.putConstraint(SpringLayout.WEST, iwadLabel, 2, SpringLayout.WEST, paths);
			sl.putConstraint(SpringLayout.NORTH, iwadLabel, 8, SpringLayout.SOUTH, programLabel);
			sl.putConstraint(SpringLayout.WEST, iwadScrollPane, 0, SpringLayout.WEST, iwadLabel);
			sl.putConstraint(SpringLayout.NORTH, iwadScrollPane, 2, SpringLayout.SOUTH, iwadLabel);
			sl.putConstraint(SpringLayout.WEST, addIwad, 50, SpringLayout.WEST, iwadScrollPane);
			sl.putConstraint(SpringLayout.NORTH, addIwad, 2, SpringLayout.SOUTH, iwadScrollPane);
			sl.putConstraint(SpringLayout.WEST, removeIwad, 2, SpringLayout.EAST, addIwad);
			sl.putConstraint(SpringLayout.NORTH, removeIwad, 0, SpringLayout.NORTH, addIwad);
			sl.putConstraint(SpringLayout.WEST, pwadLabel, 2, SpringLayout.WEST, paths);
			sl.putConstraint(SpringLayout.NORTH, pwadLabel, 2, SpringLayout.SOUTH, addIwad);
			sl.putConstraint(SpringLayout.WEST, pwadScrollPane, 0, SpringLayout.WEST, pwadLabel);
			sl.putConstraint(SpringLayout.NORTH, pwadScrollPane, 2, SpringLayout.SOUTH, pwadLabel);
			sl.putConstraint(SpringLayout.WEST, addPwad, 50, SpringLayout.WEST, pwadScrollPane);
			sl.putConstraint(SpringLayout.NORTH, addPwad, 2, SpringLayout.SOUTH, pwadScrollPane);
			sl.putConstraint(SpringLayout.WEST, removePwad, 2, SpringLayout.EAST, addPwad);
			sl.putConstraint(SpringLayout.NORTH, removePwad, 0, SpringLayout.NORTH, addPwad);
			sl.putConstraint(SpringLayout.WEST, downloadLabel, 2, SpringLayout.WEST, paths);
			sl.putConstraint(SpringLayout.NORTH, downloadLabel, 2, SpringLayout.SOUTH, addPwad);
			sl.putConstraint(SpringLayout.WEST, download, 5, SpringLayout.WEST, downloadLabel);
			sl.putConstraint(SpringLayout.NORTH, download, 4, SpringLayout.SOUTH, downloadLabel);
			sl.putConstraint(SpringLayout.WEST, downloadButton, 2, SpringLayout.EAST, download);
			sl.putConstraint(SpringLayout.NORTH, downloadButton, -3, SpringLayout.NORTH, download);

			paths.add(programLabel);
			paths.add(program);
			paths.add(programButton);
			paths.add(iwadLabel);
			paths.add(iwadScrollPane);
			paths.add(addIwad);
			paths.add(removeIwad);
			paths.add(pwadLabel);
			paths.add(pwadScrollPane);
			paths.add(addPwad);
			paths.add(removePwad);
			paths.add(downloadLabel);
			paths.add(download);
			paths.add(downloadButton);
		}
		sl = new SpringLayout();
		JPanel settings = new JPanel(sl);
		{
			JLabel masterLabel = new JLabel("Master Server:");
			JLabel masterPortLabel = new JLabel(":");
			master = new JTextField(13);
			masterPort = new JFormattedTextField(new DecimalFormat("00000")); //I need to fix this I think...
			masterPort.setColumns(4);
			masterPort.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
			JLabel timeoutLabel = new JLabel("Maximum server wait time (ms):");
			timeout = new JSpinner(new SpinnerNumberModel(0, 0, 6000, 1));
			adaptToSystem = new JCheckBox("Try to use system look and feel");
			debianFix = new JCheckBox("Query in two passes (not recommended)");

			sl.putConstraint(SpringLayout.WEST, masterLabel, 2, SpringLayout.WEST, settings);
			sl.putConstraint(SpringLayout.NORTH, masterLabel, 5, SpringLayout.NORTH, settings);
			sl.putConstraint(SpringLayout.WEST, master, 2, SpringLayout.EAST, masterLabel);
			sl.putConstraint(SpringLayout.NORTH, master, 0, SpringLayout.NORTH, masterLabel);
			sl.putConstraint(SpringLayout.WEST, masterPortLabel, 1, SpringLayout.EAST, master);
			sl.putConstraint(SpringLayout.NORTH, masterPortLabel, 0, SpringLayout.NORTH, masterLabel);
			sl.putConstraint(SpringLayout.WEST, masterPort, 1, SpringLayout.EAST, masterPortLabel);
			sl.putConstraint(SpringLayout.NORTH, masterPort, 0, SpringLayout.NORTH, master);
			sl.putConstraint(SpringLayout.WEST, timeoutLabel, 2, SpringLayout.WEST, settings);
			sl.putConstraint(SpringLayout.NORTH, timeoutLabel, 8, SpringLayout.SOUTH, masterLabel);
			sl.putConstraint(SpringLayout.WEST, timeout, 2, SpringLayout.EAST, timeoutLabel);
			sl.putConstraint(SpringLayout.NORTH, timeout, 0, SpringLayout.NORTH, timeoutLabel);
			sl.putConstraint(SpringLayout.WEST, adaptToSystem, 2, SpringLayout.WEST, settings);
			sl.putConstraint(SpringLayout.NORTH, adaptToSystem, 5, SpringLayout.SOUTH, timeoutLabel);
			sl.putConstraint(SpringLayout.WEST, debianFix, 2, SpringLayout.WEST, settings);
			sl.putConstraint(SpringLayout.NORTH, debianFix, 2, SpringLayout.SOUTH, adaptToSystem);

			settings.add(masterLabel);
			settings.add(master);
			settings.add(masterPortLabel);
			settings.add(masterPort);
			settings.add(timeoutLabel);
			settings.add(timeout);
			settings.add(adaptToSystem);
			settings.add(debianFix);
		}
		settings.setBorder(new TitledBorder(new EtchedBorder(), "Settings"));

		//The button bar is for the "ok", "apply", and "cancel" buttons.
		JPanel buttonBar = new JPanel();
		{
			JButton ok = new JButton("Ok");
			JButton apply = new JButton("Apply");
			JButton cancel = new JButton("Cancel");
			ok.setActionCommand("Ok");
			ok.addActionListener(this);
			apply.setActionCommand("Apply");
			apply.addActionListener(this);
			cancel.setActionCommand("Cancel");
			cancel.addActionListener(this);
			buttonBar.add(ok);
			buttonBar.add(apply);
			buttonBar.add(cancel);
		}

		JPanel layoutPanel = new JPanel(new GridLayout(1, 2));
		layoutPanel.add(paths);
		layoutPanel.add(settings);
		frame.add(layoutPanel);
		frame.add(buttonBar, BorderLayout.SOUTH);
		frame.setVisible(true);
		init();
	}

	public void init()
	{
		//Fill the dialog with values from your config.
		program.setText(Config.skulltagPath);
		download.setText(Config.downloadDir);
		iwadList.setListData(Config.iwadPath.toArray());
		pwadList.setListData(Config.pwadPath.toArray());
		master.setText(Config.masterServerAddress);
		masterPort.setText(new Integer(Config.masterServerPort).toString());
		timeout.getModel().setValue(Config.waitingForServerTimeOut);
		adaptToSystem.setSelected(Config.adaptToSystem);
		debianFix.setSelected(Config.debianFix);
	}

	public void actionPerformed(ActionEvent e)
	{
		if(e.getActionCommand().equals("Ok") || e.getActionCommand().equals("Apply"))
		{
			Config.skulltagPath = program.getText();
			Config.downloadDir = download.getText();
			Config.masterServerAddress = master.getText();
			int port = Integer.parseInt(masterPort.getText());
			if(port < 0) //Make sure we have a valid port number.
				port = 0;
			else if(port > 65535)
				port = 65535;
			Config.masterServerPort = port;
			Config.waitingForServerTimeOut = ((Integer) timeout.getModel().getValue()).intValue();
			Config.adaptToSystem = adaptToSystem.isSelected();
			Config.debianFix = debianFix.isSelected();
		}
		if(e.getActionCommand().equals("Ok") || e.getActionCommand().equals("Cancel"))
		{
			open = false;
			frame.setVisible(false);
			return;
		}
		if(e.getActionCommand().indexOf("Browse") == 0)
		{
			fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
			fc.showOpenDialog(frame);
			if(e.getActionCommand().indexOf("skulltagPath") != -1 && fc.getSelectedFile() != null)
				program.setText(fc.getSelectedFile().getPath());
			return;
		}
		if(e.getActionCommand().indexOf("Add") == 0) //This along with removing do not require being applied
		{
			fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			fc.showOpenDialog(frame);
			if(fc.getSelectedFile() == null)
				return;
			if(e.getActionCommand().indexOf("iwadPath") != -1)
			{
				Config.iwadPath.add(fc.getSelectedFile().getPath() + "/");
				iwadList.setListData(Config.iwadPath.toArray());
			}
			else if(e.getActionCommand().indexOf("pwadPath") != -1)
			{
				Config.pwadPath.add(fc.getSelectedFile().getPath() + "/");
				pwadList.setListData(Config.pwadPath.toArray());
			}
			return;
		}
		if(e.getActionCommand().equals("Remove iwadPath"))
		{
			if(iwadList.getSelectedIndex() == -1)
				return;
			Config.iwadPath.remove(iwadList.getSelectedIndex());
			iwadList.setListData(Config.iwadPath.toArray());
			return;
		}
		if(e.getActionCommand().equals("Remove pwadPath"))
		{
			if(pwadList.getSelectedIndex() == -1)
				return;
			Config.pwadPath.remove(pwadList.getSelectedIndex());
			pwadList.setListData(Config.pwadPath.toArray());
			return;
		}
		if(e.getActionCommand().equals("DirBrowse downloadDir"))
		{
			fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			fc.showOpenDialog(frame);
			if(fc.getSelectedFile() != null)
				download.setText(fc.getSelectedFile().getPath() + "/");
			return;
		}
	}
	public void windowClosing(WindowEvent e)
	{
		open = false;
	}
}