/* ServerListModel.java
--------------------------------------------------------------------------------
Table Model for the server list.
*/

package gui;

import main.Main;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;

import static main.Main.servers;

public class ServerListModel extends AbstractTableModel implements MouseListener
{
	private static final int COLUMNS = 8;
	public static final int COL_PLAYERS = 0;
	public static final int COL_PING = 1;
	public static final int COL_NAME = 2;
	public static final int COL_IP = 3;
	public static final int COL_MAP = 4;
	public static final int COL_IWAD = 5;
	public static final int COL_PWAD = 6;
	public static final int COL_GAMETYPE = 7;

	private int lastSelectedRow = 0;
	private long lastClick = 0;

	public String getColumnName(int col)
	{
		switch(col)
		{
			case COL_PLAYERS: return "Players";
			case COL_PING: return "Ping";
			case COL_NAME: return "Server Name";
			case COL_IP: return "IP Address";
			case COL_MAP: return "Map";
			case COL_IWAD: return "Game";
			case COL_PWAD: return "Wad(s)";
			case COL_GAMETYPE: return "Gamemode";
			default: return "Unknown";
		}
	}
	public int getRowCount()
	{
		return servers.size();
	}
	public int getColumnCount()
	{
		return COLUMNS;
	}
	public boolean isCellEditable(int row, int col)
	{
		lastSelectedRow = row;
		return false;
	}
	public Object getValueAt(int row, int col)
	{
		switch(col)
		{
			case COL_PLAYERS: return servers.get(row).getPlayersImage();
			case COL_PING: return servers.get(row).ping();
			case COL_NAME: return servers.get(row).name();
			case COL_IP: return servers.get(row).address();
			case COL_MAP: return servers.get(row).map();
			case COL_IWAD: return servers.get(row).game();
			case COL_PWAD: return servers.get(row).wadList();
			case COL_GAMETYPE: return servers.get(row).modeName();
		}
		return "Unkown Column";
	}
	public void setValueAt(Object value, int row, int col)
	{
	}
	public Class getColumnClass(int c)
	{
		return getValueAt(0, c).getClass();
	}

	//Mouse listening
	public void mouseClicked(MouseEvent e)
	{
		int button = e.getButton();  
		if(button == MouseEvent.BUTTON1)
		{
			if(System.currentTimeMillis() - lastClick < 300)
			{
				Main.refreshServer(lastSelectedRow);
				Main.join(servers.get(lastSelectedRow));
			}
			lastClick = System.currentTimeMillis();
		}
		if(button == MouseEvent.BUTTON2)
		{
			lastClick = 0;
		}
		if(button == MouseEvent.BUTTON3)
		{
			lastClick = 0;
			Main.refreshServer(lastSelectedRow);
		}
	}
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mousePressed(MouseEvent e)
	{
		if(e.getButton() != MouseEvent.BUTTON1)
		{
			//fake button1 press.
			try
			{
				Robot robot = new Robot();
				robot.mousePress(InputEvent.BUTTON1_MASK);
				robot.mouseRelease(InputEvent.BUTTON1_MASK);
			}
			catch(Exception exc)
			{
				exc.printStackTrace();
			}
		}
	}
	public void mouseReleased(MouseEvent e){}
}