/* Config.java
--------------------------------------------------------------------------------
Reads in the config file.
*/

package main;

import main.*;

import java.io.*;
import java.util.*;

import static main.Main.crash;

public class Config
{
	public static LinkedList<String> iwadPath = new LinkedList<String>();
	public static LinkedList<String> pwadPath = new LinkedList<String>();
	public static LinkedList<String> wadSearchURL = new LinkedList<String>();
	public static String masterServerAddress = "skulltag.servegame.com";
	public static String skulltagPath = "";
	public static String downloadDir = "./";
	public static boolean adaptToSystem = true;
	public static boolean debugMode = false;
	public static boolean debianFix = false;
	public static int masterServerPort = 15300;
	public static int waitingForServerTimeOut = 300;
	public static int windowX = 0;
	public static int windowY = 0;
	public static int windowWidth = 640;
	public static int windowHeight = 400;

	private static String str = "\"[a-zA-Z0-9\\\\:=_. /~\\-]*\"";
	private static String file = "browser.cfg";

	//CMD Line parser
	public static void parseCMDLine(String[] args)
	{
		try
		{
			for(int i = 0;i < args.length;i++)
			{
				if(args[i].equals("-config"))
					file = args[++i];
				if(args[i].equals("-debug"))
					debugMode = true;
			}
		}
		catch(ArrayIndexOutOfBoundsException e)
		{
			crash("Malformed command line.");
		}
	}

	//Config file reader
	public static void read(String file)
	{
		Config.file = file;
		try
		{
			Scanner sc = new Scanner(new FileInputStream(file));
			while(sc.hasNext("[a-zA-Z0-9_]*"))
			{
				String var = sc.next("[a-zA-Z0-9_]*");
				if(var.equals("masterServerAddress"))
					masterServerAddress = string(sc.next(str));
				else if(var.equals("masterServerPort"))
					masterServerPort = sc.nextInt();
				else if(var.equals("skulltagPath"))
					skulltagPath = string(sc.next(str));
				else if(var.equals("adaptToSystem"))
					adaptToSystem = sc.nextBoolean();
				else if(var.equals("waitingForServerTimeOut"))
					waitingForServerTimeOut = sc.nextInt();
				else if(var.equals("windowX"))
					windowX = sc.nextInt();
				else if(var.equals("windowY"))
					windowY = sc.nextInt();
				else if(var.equals("windowWidth"))
					windowWidth = sc.nextInt();
				else if(var.equals("windowHeight"))
					windowHeight = sc.nextInt();
				else if(var.equals("downloadDir"))
					downloadDir = string(sc.next(str));
				else if(var.equals("iwadPath"))
					iwadPath.add(string(sc.next(str)));
				else if(var.equals("pwadPath"))
					pwadPath.add(string(sc.next(str)));
				else if(var.equals("wadSearchURL"))
					wadSearchURL.add(string(sc.next(str)));
				else if(var.equals("debianFix"))
					debianFix = sc.nextBoolean();
				else
					crash("Unknown config variable: " + var);
			}
			sc.close();
		}
		catch(FileNotFoundException e) //Can't find the config?  No problem, just create one.
		{
			//Of course we'll want to set some default FindWad paths
			wadSearchURL.add("http://relentlessbeating.com/wads/");
			wadSearchURL.add("http://www.coffeenet.org/~vulture/wads/");
		}
		catch(Exception e)
		{
			e.printStackTrace();
			crash("Unable to read config.");
		}
	}
	public static void store()
	{
		try
		{
			byte[] out = (
			   "masterServerAddress \"" + masterServerAddress + "\"\n" +
			   "masterServerPort " + masterServerPort + "\n" +
			   "skulltagPath \"" + skulltagPath + "\"\n" +
			   "adaptToSystem " + adaptToSystem + "\n" +
			   "waitingForServerTimeOut " + waitingForServerTimeOut + "\n" +
			   "windowX " + windowX + "\n" +
			   "windowY " + windowY + "\n" +
			   "windowWidth " + windowWidth + "\n" +
			   "windowHeight " + windowHeight + "\n" +
			   "downloadDir \"" + downloadDir + "\"\n" +
			   "debianFix " + debianFix + "\n" +
			   storePaths()
			).getBytes();
			FileOutputStream outs = new FileOutputStream(file);
			outs.write(out, 0, out.length);
			outs.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			crash("Error while saving config.");
		}
	}
	public static String getLastConfigFile()
	{
		return file;
	}
	private static String storePaths()
	{
		String out = "";
		for(String path : iwadPath)
		{
			out +=
			   "iwadPath \"" + path + "\"\n";
		}
		for(String path : pwadPath)
		{
			out +=
			   "pwadPath \"" + path + "\"\n";
		}
		for(String url : wadSearchURL)
		{
			out +=
			   "wadSearchURL \"" + url + "\"\n";
 		}
		return out;
	}
	private static String string(String in)
	{
		return in.substring(1, in.length()-1);
	}
}