/* Main.java
--------------------------------------------------------------------------------
Entry point of the program.
*/

package main;

import main.wad.*;
import client.*;
import gui.*;
import huffman.*;

import java.awt.*;
import java.awt.datatransfer.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;

public class Main
{
	public static final String version = "0.2 Alpha (r15)";
	private static Thread guiThread;
	private static MasterClient mc;
	private static int standardQuery = Server.SQF_NAME + Server.SQF_URL + Server.SQF_MAPNAME + Server.SQF_MAXCLIENTS + Server.SQF_MAXPLAYERS + Server.SQF_PWADS + Server.SQF_GAMETYPE + Server.SQF_IWAD + Server.SQF_NUMPLAYERS + Server.SQF_PLAYERDATA;
	public static ArrayList<Server> servers;
	public static GUI gui;
	public static Thread cmdThread;
	public static CommandThread cmd;

	public static void main(String[] args)
	{
		Config.parseCMDLine(args);
		Config.read(Config.getLastConfigFile());
		servers = new ArrayList<Server>();
		mc = new MasterClient(Config.masterServerAddress, Config.masterServerPort);
		gui = new GUI();
		guiThread = new Thread(gui);
		guiThread.start();
		cmd = new CommandThread(); //Not sure I like this implementation
		cmdThread = new Thread(cmd);
		if(!Config.debugMode)
			cmd.setCommand(CommandThread.CMD_FULLREFRESH);
	}
	public static void crash(String text)
	{
		JOptionPane.showMessageDialog(new JFrame(), text, "Fatal Error", JOptionPane.ERROR_MESSAGE);
		System.exit(1);
	}
	public static void error(String text)
	{
		JOptionPane.showMessageDialog(new JFrame(), text, "Error", JOptionPane.ERROR_MESSAGE);
	}
	public static void join(Server server)
	{
		if(server.numberOfPlayers(false) == server.maxClients())
		{
			if(Config.debugMode)
				System.out.println("Max Clients: " + server.maxClients() + " Num Players: " + server.numberOfPlayers(false) + " Max Players: " + server.maxPlayers());
			error("Server full.");
			return;
		}
		try
		{
			if(new File(Config.skulltagPath).exists())
			{
				try
				{
					ProcessBuilder pb = new ProcessBuilder(server.launchString());
					if(!Config.debugMode)
						pb.start();
					else
						System.out.println("Joined: " + server.name());
				}
				catch(IWadNotFoundException e)
				{
					error("Unable to locate " + e.getMessage() + ".\nYou'll probably have to buy this game.");
				}
				catch(PWadNotFoundException e)
				{
					if(!server.url().equals(""))
						error("Unable to locate " + e.getMessage() + ".\nYou might find it at: \"" + server.url() + "\".");
					else
						error("Unable to locate " + e.getMessage() + ".\nLook around the internet for this file.");
					FindWad.findWad(new WadFile(e.getMessage()));
				}
			}
			else
				error("Unable to start Skulltag.  Check your path.");
		}
		catch(Exception e)
		{
			error("You need to give Java permission to start a new runtime process.");
		}
	}
	public static void refreshFull()
	{
		Server[] list = new Server[0];
		servers = new ArrayList<Server>();
		try
		{
			list = mc.refresh(); //This goes through pretty fast so lets not update the list on the fly just yet
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return;
		}
		if(mc.getLastResponse() != MasterClient.RESPONSE_OK || list == null)
			return;
		int availableServers = list.length;
		for(int i = 0;i < list.length;i++)
		{
			int response = list[i].refresh(Config.debianFix ? standardQuery - Server.SQF_GAMETYPE : standardQuery);
			if(list[i].isResponsive())
			{
				servers.add(list[i]);
				gui.setQueryProgress(i+1, list.length);
				gui.updateList(true, servers.size()-1);
			}
			else
			{
				availableServers--;
				gui.setQueryProgress(i+1, list.length);
			}
		}
		if(Config.debianFix) //Now we query just the gametype
		{
			for(int i = 0;i < servers.size();i++)
			{
				servers.get(i).refresh(Server.SQF_GAMETYPE);
				gui.setQueryProgress(i+1, servers.size(), servers.size());
				gui.updateList(false, i);
			}
		}
		gui.setQueryProgress(list.length, list.length, availableServers);
	}
	public static void refreshAll()
	{
		for(int i = 0;i < servers.size();i++)
		{
			servers.get(i).refresh(Config.debianFix ? standardQuery - Server.SQF_GAMETYPE : standardQuery);
			gui.setQueryProgress(i+1, servers.size());
			gui.updateList(false, i);
		}
		if(Config.debianFix) //Now we query just the gametype
		{
			for(int i = 0;i < servers.size();i++)
			{
				servers.get(i).refresh(Server.SQF_GAMETYPE);
				gui.setQueryProgress(i+1, servers.size(), servers.size());
				gui.updateList(false, i);
			}
		}
		gui.setQueryProgress(servers.size(), servers.size(), servers.size());
	}
	public static void refreshServer(int server)
	{
		//If we have the incomplete packet bug work around enabled we will not update the gametype.
		servers.get(server).refresh(Config.debianFix ? standardQuery - Server.SQF_GAMETYPE : standardQuery);
		gui.updateList(false, server);
	}
	public static int countPlayers()
	{
		int num = 0;
		for(Server server : servers)
			num += server.numberOfPlayers(true);
		return num;
	}

	public static int unsigned(byte in)
	{
		if(in < 0)
			return 256 + in;
		else
			return in;
	}
	public static void copyToClipboard(String str)
	{
		Clipboard cb = Toolkit.getDefaultToolkit().getSystemSelection();
		StringSelection ss = new StringSelection(str);
		cb.setContents(ss, ss);
	}
}